package com.bgy.sms.channel.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bgy.sms.channel.bgy.config.BgySMSConfig;
import com.bgy.sms.channel.dmHub.config.DmHubConfig;
import com.bgy.sms.channel.dto.*;
import com.bgy.sms.config.ResponseCode;
import com.bgy.sms.repository.domain.SmsTemplateInfo;
import com.bgy.sms.service.MessageService;
import com.bgy.sms.service.SmsTemplateService;
import com.bgy.util.HttpUtil;
import com.bgy.util.Md5Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@SuppressWarnings("Duplicates")
@RestController
@RequestMapping()
public class DmHubApi {

    private static final Logger log = LoggerFactory.getLogger(DmHubApi.class);

    @Autowired
    private MessageService messageService;
    @Autowired
    private SmsTemplateService smsTemplateService;

    @GetMapping("/ping")
    public String ping() {
        log.info("====sms ping=====");
        return "pong";
    }


    /**
     * 创建短信模板
     *
     * @return
     */
    @PostMapping("/sms/template")
    public DmHubResponse template(@RequestBody DmHubTemplateRequest params, String appId, String timestamp, String signature) {
        log.info("**********创建模板接口入参*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(params), appId, timestamp, signature);
        DmHubResponse response;
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                return new DmHubResponse("555", "接口请求签名校验不通过");
            }
            response = messageService.createTemplate(params);


        } catch (Exception e) {
            log.error("创建模板短信异常", e);
            response = new DmHubResponse("999", "创建模板短信异常");
        }
        log.info("**********创建模板接口出参*******：{}", JSONObject.toJSONString(response));
        return response;
    }

    /**
     * 发送短信验证码
     *
     * @return
     */
    @PostMapping("/sms/verifyCode")
    public DmHubResponse verifyCode(@RequestBody DmHubCodeRequest params, String appId, String timestamp, String signature) {
        log.info("**********发送短信验证码*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(params), appId, timestamp, signature);
        DmHubResponse response;
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                return new DmHubResponse("555", "接口请求签名校验不通过");
            }
            response = messageService.sendCode(params);
        } catch (Exception e) {
            log.error("发送验证码短信异常", e);
            response = new DmHubResponse("999", "发送验证码短信异常");
        }
        log.info("**********发送短信验证码出参*******：{}", JSONObject.toJSONString(response));
        return response;
    }


    /**
     * 批量发送（通知或营销类）
     *
     * @return
     */
    @PostMapping("/sms/batch")
    public DmHubResponse batch(@RequestBody DmHubBatchSendRequest request, String appId, String timestamp, String signature) {
        log.info("**********批量发送入参*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(request), appId, timestamp, signature);
        DmHubResponse response;
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                return new DmHubResponse("555", "接口请求签名校验不通过");
            }
            response = messageService.batchSendOneByOne(request);
            response.setCode(response.getError().getString("errorCode"));
        } catch (Exception e) {
            log.error("发送批量短信异常", e);
            response = new DmHubResponse(ResponseCode.SYSTEM_ERROR);
        }
        log.info("**********批量发送出参*******：{}", JSONObject.toJSONString(response));
        return response;
    }

    public boolean checkSignature(String timestamp, String signature) {
        String signStr = DmHubConfig.appId + DmHubConfig.appSecret + timestamp;
        String encrypt = Md5Util.encrypt(signStr);
        log.info("待签名串：" + encrypt);
        if (!encrypt.equals(signature)) {
            log.error("签名不一致，插件生成签名：{}", encrypt);
            return false;
        }
        return true;

    }

    /**
     * 发送单条（通知或营销类）
     *
     * @return
     */
    @RequestMapping("/sms/send")
    public DmHubResponse send(@RequestBody DmHubSendRequest request, String appId, String timestamp, String signature) {
        log.info("**********单条发送入参*******：{},\r\n appId:{},timestamp:{},signature:{}", JSONObject.toJSONString(request), appId, timestamp, signature);
        DmHubResponse response;
        try {
            boolean checkResult = this.checkSignature(timestamp, signature);
            if (!checkResult) {
                return new DmHubResponse("555", "签名校验不通过");
            }
            response = messageService.send(request);
            response.setCode(response.getError().getString("errorCode"));
        } catch (Exception e) {
            log.error("发送单条短信异常", e);
            response = new DmHubResponse(ResponseCode.SYSTEM_ERROR);
        }
        log.info("**********单条发送出参*******：{}", JSONObject.toJSONString(response));
        return response;
    }

    public static void main(String[] args) {

        String z = "{\"err\":\"成功\",\"package\":\"{\\\"data\\\":\\\"8fff70a0-d751-4d2b-b4c6-17edec541ec7\\\"}\",\"ret\":\"0\"} ";
        JSONObject retJson = JSONObject.parseObject(z);
        String data = retJson.getString("package");
        JSONObject retJson2 = JSONObject.parseObject(data);
        String data2 = retJson2.getString("data");
        //String retStr = HttpUtil.sendPost(BgySMSConfig.url, JSONObject.toJSONString(z));
        System.out.println(data2);

    }
}
